# Name: Lines to Arc
# Version: 1.0
# SU Version: Developed on SU Pro 6, should be compatible 5 and later.
# Description: This scipt converts lines to an arc curve, even after the arc has been exploded in SketchUp.
# Usage: Select two or more line segments that used to be part of an arc object in SketchUp.  Then run the plugin from Plugins>ni>
# License: free.
# History:
#   1.0:: 2009-08-13   * First release
# Special thanks: Chris Fullmer the great developer.

require 'sketchup.rb'
module Ni_lines2arc
def Ni_lines2arc.main
	mod = Sketchup.active_model
	sel = mod.selection
	sel.each {|i| sel.remove(i) unless i.typename=="Edge"}
	if sel.length < 2
		UI.messagebox "Select 2 or more edges to convert to an arc."
		return
	end
	ll = []
	sel.each {|i| ll.push(i.length)}
	unless ll == [ll[0]]*sel.length
		UI.messagebox "All of edges must have the same length."
		return
	end
	ps = []
	sel.each {|i| ps.push(i.start.position,i.end.position)}
	type = "circle"
	mm = []
	ps.each do |i|
		if ps.index(i)==ps.rindex(i)
			type = "arc"
			mm.push(i)
		end
	end
	ents = mod.active_entities
	seg1 = sel[0]
	seg2 = sel[1]
	s1p1 = seg1.start.position
	s1p2 = seg1.end.position
	s2p1 = seg2.start.position
	s2p2 = seg2.end.position
	v1 = seg1.line[1]
	v2 = seg2.line[1]
	if v1.parallel? v2
		if sel.length > 2
			seg2 = sel[2]
			s2p1 = seg2.start.position
			s2p2 = seg2.end.position
			v2 = seg2.line[1]
		else
			UI.messagebox "2 edges are parallel."
			return
		end
	end
	mp1 = Geom.linear_combination 0.5, s1p1, 0.5, s1p2
	mp2 = Geom.linear_combination 0.5, s2p1, 0.5, s2p2
	vn = v1.cross v2
	v3 = v1.cross vn
	v4 = v2.cross vn
	l1 = [mp1, v3]
	l2 = [mp2, v4]
	cp = Geom.intersect_line_line(l1, l2)
	if cp == nil
		UI.messagebox "These segments does not have a centerpoint."
		return
	end
	mod.start_operation "Lines to Arc"
	vx = cp.vector_to s1p1
	rad = cp.distance s1p1
	sn = sel.length
	grp = ents.add_group
	ents2 = grp.entities
	if type == "circle"
		arc = ents2.add_arc cp, vx, vn, rad, 0, Math::PI*2, sn
	else # type == "arc"
		vmm = [(cp.vector_to mm[0]),(cp.vector_to mm[1])]
		as = (cp.vector_to ps[0]).angle_between (cp.vector_to ps[1])
		arc = ents2.add_arc cp, vmm[0], vn, rad, 0, as*sn, sn
		ps2 = []
		arc.each {|i| ps2.push(i.start.position,i.end.position)}
		ps2.each do |i|
			next if ps.index(i)
			ents2.erase_entities arc
			arc = ents2.add_arc cp, vmm[1], vn, rad, 0, as*sn, sn
			break
		end
	end
	arc.each {|i| i.soft=true } if seg1.soft?
	arc.each {|i| i.smooth=true } if seg1.smooth?
	arc.each {|i| i.hidden=true } if seg1.hidden?
	grp.explode
	sel.clear
	mod.commit_operation
end # def
end # module

if( $submenu )
	organizer = $submenu 
	organizer.add_item("Lines to Arc") { Ni_lines2arc.main  } unless file_loaded?(__FILE__)
else
	$ni_tools_menu = UI.menu("Plugins").add_submenu("ni") unless file_loaded?('ni_menu_loader')
	$ni_tools_menu.add_item("Lines to Arc") { Ni_lines2arc.main  } unless file_loaded?(__FILE__)
end

file_loaded('ni_menu_loader')
file_loaded(__FILE__)
